<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\Rma\Block\Returns;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\Registry;
use Magento\Rma\Helper\Data;
use Magento\Framework\App\Http\Context;
use Magento\Customer\Helper\View as MagentoView;
use Magento\Customer\Helper\Session\CurrentCustomer;
use Magento\Rma\Model\Item\FormFactory;
use Magento\Rma\Model\ItemFactory;
use Magento\Rma\Model\ResourceModel\Rma\Status\History\CollectionFactory;
use Magento\Rma\Model\ResourceModel\Item\CollectionFactory as ItemCollectionFactory;
use Magento\Eav\Model\Config;
use Magento\Eav\Model\Form\Factory;
use Magento\Framework\Data\Collection\ModelFactory;
use Magento\Framework\View\Element\Template\Context as TemplateContext;

class View extends \Magento\Rma\Block\Returns\View
{

    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;

    const RMA_CUSTOM_CARRIER = 'filoblu_disablerma_section/rma_group/rma_option_carrier';


    /**
     * View constructor.
     * @param TemplateContext $context
     * @param ModelFactory $modelFactory
     * @param Factory $formFactory
     * @param Config $eavConfig
     * @param ItemCollectionFactory $itemsFactory
     * @param CollectionFactory $historiesFactory
     * @param ItemFactory $itemFactory
     * @param FormFactory $itemFormFactory
     * @param CurrentCustomer $currentCustomer
     * @param MagentoView $customerView
     * @param Context $httpContext
     * @param Data $rmaData
     * @param Registry $registry
     * @param CustomerRepositoryInterface $customerRepository
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param OrderRepositoryInterface $orderRepository
     * @param ProductRepositoryInterface $productRepository
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param array $data
     */
    public function __construct(TemplateContext $context,
                                ModelFactory $modelFactory,
                                Factory $formFactory,
                                Config $eavConfig,
                                ItemCollectionFactory $itemsFactory,
                                CollectionFactory $historiesFactory,
                                ItemFactory $itemFactory,
                                FormFactory $itemFormFactory,
                                CurrentCustomer $currentCustomer,
                                MagentoView $customerView,
                                Context $httpContext,
                                Data $rmaData,
                                Registry $registry,
                                CustomerRepositoryInterface $customerRepository,
                                RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
                                OrderRepositoryInterface $orderRepository,
                                ProductRepositoryInterface $productRepository,
                                ChangeSizeAttributeHelper $changeSizeAttributeHelper,
                                array $data = [])
    {
        parent::__construct($context, $modelFactory, $formFactory, $eavConfig, $itemsFactory, $historiesFactory, $itemFactory, $itemFormFactory, $currentCustomer, $customerView, $httpContext, $rmaData, $registry, $customerRepository, $data);
        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->orderRepository = $orderRepository;
        $this->productRepository = $productRepository;
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
    }

    /**
     * @param $myItemId
     * @return array
     */
    public function getChangeSizeItems($myItemId){

        try{
            $newSize = array_fill_keys(['label','value'],'');
            $changeSizeOrderId  = $this->rmaExtensionAttributesRepository->getChangeSizeOrderId($myItemId);
            if($changeSizeOrderId){
                $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                $items = $changeSizeOrder->getItems();
                foreach ($items as $item) {
                    $product = $this->productRepository->get($item->getSku());
                    $newSize ['label'] = $item->getProduct()->getName();
                    $newSize ['value'] = $product->getResource()->getAttribute('size')->getFrontend()->getValue($product);
                }
            }else{
                $newSize["value"] = "No";
            }
            return $newSize;
        }catch (\Exception $e){
            $this->_logger->critical($e->getMessage());
            return array_fill_keys(['label','value'],'');
        }
    }


    /**
     * Gets attribute value by rma item id and attribute code
     *
     * @param  int $itemId
     * @param  string $attributeCode
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getAttributeValue($itemId, $attributeCode)
    {

        if (empty($this->_realValueAttributes)) {
            $this->_realValueAttributes = $this->_getAdditionalData();
        }

        if (!isset($this->_realValueAttributes[$itemId][$attributeCode])) {
            return false;
        }

        if (!empty($this->_realValueAttributes[$itemId][$attributeCode]['html'])) {
            $html = $this->_realValueAttributes[$itemId][$attributeCode]['html'];
        } else {
            if($attributeCode == ChangeSizeAttributeHelper::ORDER_STATUS_CHANGESIZE_CODE ){
                $html = $this->escapeHtml($this->getChangeSizeItems($itemId)['value']);
            }else if($this->_getAdditionalData()[$itemId][$attributeCode]['value'] == $this->changeSizeAttributeHelper->getChangeSizeLabelByStoreId($this->_storeManager->getStore()->getId())){
                $html = "<span>". $this->_getAdditionalData()[$itemId][$attributeCode]['value']."</span><br/>".$this->getItems()->getItems()[$itemId]->getProductName().
                        "<dl class=\"item options\"><dt>".$this->getItemOptions($this->getItems()->getItems()[$itemId])[0]['label'] ."</dt><dd>". $this->getChangeSizeItems($itemId)['value']."</dd></dl>";;
            }else{
                $html = $this->escapeHtml($this->_realValueAttributes[$itemId][$attributeCode]['value']);
            }

        }
        return $html;
    }

    /**
     * @return bool
     */
    public function isCustomCarrierEnabled(){
        return $this->_scopeConfig->getValue(self::RMA_CUSTOM_CARRIER) ?? false;
    }

    /**
     * @param array|string $data
     * @param null $allowedTags
     * @return array|string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function escapeHtml($data, $allowedTags = null)
    {
        if(strstr($data,$this->changeSizeAttributeHelper->getChangeSizeLabelByStoreId($this->_storeManager->getStore()->getId())))
        {
            return $data;
        }
        return parent::escapeHtml($data, $allowedTags);
    }

}
